#include <Wire.h>
#include "quaternion.h"
#include "SparkFun_BNO080_Arduino_Library.h"

BNO080 myIMU;

#define X_AXIS_SENSITIVITY  50
#define Y_AXIS_SENSITIVITY  50

void setup()
{
  Serial.begin(115200);
  Serial.println();
  Serial.println("BNO080 Read");
  
  Wire.begin();

  if (myIMU.begin() == false)
  {
    Serial.println(F("BNO080 not detected at default I2C address. Check your jumpers and the hookup guide. Freezing..."));
    while (1)
      ;
  }

  Wire.setClock(400000);

  myIMU.enableGyroIntegratedRotationVector(1); 

  Serial.println("Started!!!");
}

imu::Quaternion prevQuat;
int dx = 0, dy = 0;

void loop()
{
  //Look for reports from the IMU
  if (myIMU.dataAvailable() == true)
  {
    float dqw = myIMU.getQuatReal(); float dqx = myIMU.getQuatI(); float dqy = myIMU.getQuatJ(); float dqz = myIMU.getQuatK();
    imu::Quaternion currQuat(dqw, dqx, dqy, dqz);
    prevQuat = currQuat;
    
    imu::Quaternion newQuat = currQuat * prevQuat.conjugate();
    newQuat.normalize();
    imu::Vector<3> euler = newQuat.toEuler(); 

    dx = getValue(euler.z() * 180 / PI, X_AXIS_SENSITIVITY, false);
    dy = getValue(euler.y() * 180 / PI, Y_AXIS_SENSITIVITY, false);

    char buffer[64];
    sprintf(buffer, "%i;%i", dx, dy);
  
    Serial.println(buffer);
  }
}

int getValue(double val, double sensitivity, bool is_rotation)
{
  constexpr double c[] = { 3e-1, 1e-1 };
  return (int)round(val * sensitivity * c[unsigned(is_rotation)]);
}
